@@ -22,7 +22,9 @@
 from __future__ import print_function
 import os
 import fnmatch
+import re
 import subprocess
+import sys
 
 def find(pattern, classPaths):
     paths = classPaths.split(os.pathsep)
@@ -32,14 +34,14 @@ def find(pattern, classPaths):
         # remove * if it's at the end of path
         if ((path is not None) and (len(path) > 0) and (path[-1] == '*')) :
             path = path[:-1]
-    
+
         for root, dirs, files in os.walk(path):
             # sort the file names so *-client always precedes *-thin-client
             files.sort()
             for name in files:
                 if fnmatch.fnmatch(name, pattern):
                     return os.path.join(root, name)
-                
+
     return ""
 
 def tryDecode(input):
@@ -50,6 +52,15 @@ def tryDecode(input):
     except:
         return input
 
+def tryQuote(unquoted_input):
+    """ Python 2/3 compatibility hack
+    """
+    try:
+        from shlex import quote as cmd_quote
+    except ImportError:
+        from pipes import quote as cmd_quote
+    return cmd_quote(unquoted_input)
+
 def findFileInPathWithoutRecursion(pattern, path):
     if not os.path.exists(path):
         return ""
@@ -78,11 +89,16 @@ def findClasspath(command_name):
 
 def setPath():
     PHOENIX_CLIENT_EMBEDDED_JAR_PATTERN = "phoenix-client-embedded-hbase-*[!s].jar"
+    PHOENIX_CLIENT_LITE_JAR_PATTERN = "phoenix-client-lite-hbase-*[!s].jar"
     PHOENIX_TRACESERVER_JAR_PATTERN = "phoenix-tracing-webapp-*-runnable.jar"
     PHOENIX_TESTS_JAR_PATTERN = "phoenix-core-*-tests*.jar"
     PHOENIX_PHERF_JAR_PATTERN = "phoenix-pherf-*[!s].jar"
-    SLF4J_BACKEND_JAR_PATTERN = "slf4j-reload4j-*[!s].jar"
-    LOGGING_JAR_PATTERN = "reload4j-*[!s].jar"
+    SLF4J_BACKEND_JAR_PATTERN = "log4j-slf4j*.jar"
+    LOGGING_JAR_PATTERN = "log4j-core*.jar"
+    LOGGING_JAR_PATTERN2 = "log4j-api*.jar"
+    LOGGING_JAR_PATTERN3 = "log4j-1.2-api*.jar"
+    LOGGING_JAR_PATTERN4 = "jul-to-slf4j*.jar"
+    LOGGING_JAR_PATTERN5 = "jcl-over-slf4j*.jar"
     SQLLINE_WITH_DEPS_PATTERN = "sqlline-*-jar-with-dependencies.jar"
 
 
@@ -129,6 +145,16 @@ def setPath():
     if phoenix_client_embedded_jar == "":
         phoenix_client_embedded_jar = find(PHOENIX_CLIENT_EMBEDDED_JAR_PATTERN, phoenix_class_path)
 
+    global phoenix_lite_jar_path
+    phoenix_lite_jar_path = os.path.join(current_dir, "..", "phoenix-client-parent" , "phoenix-client-lite", "target","*")
+
+    global phoenix_client_lite_jar
+    phoenix_client_lite_jar = find(PHOENIX_CLIENT_LITE_JAR_PATTERN, phoenix_lite_jar_path)
+    if phoenix_client_lite_jar == "":
+        phoenix_client_lite_jar = findFileInPathWithoutRecursion(PHOENIX_CLIENT_LITE_JAR_PATTERN, os.path.join(current_dir, ".."))
+    if phoenix_client_lite_jar == "":
+        phoenix_client_lite_jar = find(PHOENIX_CLIENT_LITE_JAR_PATTERN, phoenix_class_path)
+
     global phoenix_test_jar_path
     phoenix_test_jar_path = os.path.join(current_dir, "..", "phoenix-core", "target","*")
 
@@ -184,7 +210,14 @@ def setPath():
     logging_jar = os.environ.get(OVERRIDE_LOGGING)
     if logging_jar is None or logging_jar == "":
         logging_jar = findFileInPathWithoutRecursion(LOGGING_JAR_PATTERN, os.path.join(current_dir, "..","lib"))
+        logging_jar += ":"+findFileInPathWithoutRecursion(LOGGING_JAR_PATTERN2, os.path.join(current_dir, "..","lib"))
+        logging_jar += ":"+findFileInPathWithoutRecursion(LOGGING_JAR_PATTERN3, os.path.join(current_dir, "..","lib"))
+        logging_jar += ":"+findFileInPathWithoutRecursion(LOGGING_JAR_PATTERN4, os.path.join(current_dir, "..","lib"))
+        logging_jar += ":"+findFileInPathWithoutRecursion(LOGGING_JAR_PATTERN5, os.path.join(current_dir, "..","lib"))
 
+    __read_hbase_env()
+    __set_java()
+    __set_jvm_flags()
     return ""
 
 def shell_quote(args):
@@ -199,13 +232,85 @@ def shell_quote(args):
         return subprocess.list2cmdline(args)
     else:
         # pipes module isn't available on Windows
-        import pipes
-        return " ".join([pipes.quote(tryDecode(v)) for v in args])
+        return " ".join([tryQuote(tryDecode(v)) for v in args])
+
+def __set_java():
+    global java_home
+    global java
+    java_home = os.getenv('JAVA_HOME')
+    if java_home:
+        java = os.path.join(java_home, 'bin', 'java')
+    else:
+        java = 'java'
+
+
+def __read_hbase_env():
+    if os.getenv("SKIP_HBASE_ENV"):
+        return ""
+    # HBase configuration folder path (where hbase-site.xml reside) for
+    # HBase/Phoenix client side property override
+    hbase_config_path = hbase_conf_dir
+
+    # load hbase-env.??? to extract JAVA_HOME, HBASE_PID_DIR, HBASE_LOG_DIR
+    hbase_env_path = None
+    hbase_env_cmd  = None
+    if os.name == 'posix':
+        hbase_env_path = os.path.join(hbase_config_path, 'hbase-env.sh')
+        hbase_env_cmd = ['bash', '-c', 'source %s && env' % hbase_env_path]
+    elif os.name == 'nt':
+        hbase_env_path = os.path.join(hbase_config_path, 'hbase-env.cmd')
+        hbase_env_cmd = ['cmd.exe', '/c', 'call %s & set' % hbase_env_path]
+    if not hbase_env_path or not hbase_env_cmd:
+        sys.stderr.write("hbase-env file unknown on platform {}{}".format(os.name, os.linesep))
+        sys.exit(-1)
+
+    if os.path.isfile(hbase_env_path):
+        p = subprocess.Popen(hbase_env_cmd, stdout = subprocess.PIPE)
+        for x in p.stdout:
+            (k, _, v) = tryDecode(x).partition('=')
+            os.environ[k.strip()] = v.strip()
+
+    return ""
+
+
+def __set_jvm_flags():
+    global jvm_module_flags
+    jvm_module_flags = ""
+    # This should be ASCII
+    version_output = subprocess.check_output([java, "-version"], stderr=subprocess.STDOUT).decode()
+    version_output = tryDecode(version_output)
+    m = re.search(r'version\s"(\d+)\.(\d+)', version_output)
+    if (m is None):
+        # Could not find version
+        return ""
+    major = m.group(1)
+    minor = m.group(2)
+    if (major is None or minor is None):
+        #Could not identify version
+        return ""
+    if (minor == "1"):
+        major = minor
+    if (int(major) >= 11):
+        # Copied from hbase startup script
+        jvm_module_flags = "-Dorg.apache.hbase.thirdparty.io.netty.tryReflectionSetAccessible=true \
+--add-modules jdk.unsupported \
+--add-opens java.base/java.nio=ALL-UNNAMED \
+--add-opens java.base/sun.nio.ch=ALL-UNNAMED \
+--add-opens java.base/java.lang=ALL-UNNAMED \
+--add-opens java.base/jdk.internal.ref=ALL-UNNAMED \
+--add-opens java.base/java.lang.reflect=ALL-UNNAMED \
+--add-exports java.base/jdk.internal.misc=ALL-UNNAMED \
+--add-exports java.security.jgss/sun.security.krb5=ALL-UNNAMED \
+--add-exports java.base/sun.net.dns=ALL-UNNAMED \
+--add-exports java.base/sun.net.util=ALL-UNNAMED"
+    return ""
 
 def common_sqlline_args(parser):
-    parser.add_argument('-v', '--verbose', help='Verbosity on sqlline.', default='true')
-    parser.add_argument('-c', '--color', help='Color setting for sqlline.', default='true')
-    parser.add_argument('-fc', '--fastconnect', help='Fetch all schemas on initial connection', default='false')
+    parser.add_argument('-v', '--verbose', help='Verbosity on sqlline.', action="store_true")
+    parser.add_argument('-c', '--color', help='Color setting for sqlline.', action="store_true")
+    parser.add_argument('-fc', '--fastconnect',
+                        help='Fetch all schemas on initial connection',
+                        action="store_true")
 
 if __name__ == "__main__":
     setPath()
@@ -220,3 +325,6 @@ if __name__ == "__main__":
     print("sqlline_with_deps_jar:", sqlline_with_deps_jar)
     print("slf4j_backend_jar:", slf4j_backend_jar)
     print("logging_jar:", logging_jar)
+    print("java_home:", java_home)
+    print("java:", java)
+    print("jvm_module_flags:", jvm_module_flags)
